/*
 * @Author: zhangyu
 * @Date: 2021-04-17 18:37:49
 * @LastEditTime: 2021-04-28 21:23:28
 */

const HttpException = require('./exception.js')
const goCheck = require('./validate.js')
const ThinkDb = require('./thinkdb.js')
const ThinkRedis = require('./thinkredis.js')

class Controller extends ThinkDb{

    /*
    * 参数获取器
    * ctx 传入一个ctx
    * validate 控制是否开启对该控制器方法的参数校验 默认不开启
    */
    getParams(ctx,validate = false){
        let obj = {}
        if(ctx.request.method == 'GET'){
            obj = ctx.request.query
        }else if(ctx.request.method == 'POST'){
            obj = ctx.request.body
        }
        if(validate){
            try{
                let valObj = require(`@/validate/${ctx.before}.js`)
                Object.keys(valObj.rule).forEach(key => {
                    // 只验证有验证规则的参数
                    if(obj.hasOwnProperty(key)){
                        // 1. key 要验证的键
                        // 2. obj[key] 要验证的值
                        // 3. valObj.rule[key] 验证规则
                        // 4. 自定义验证提示
                        // 5. 分组验证
                        let message = ''
                        if(Object.keys(valObj.message).filter(v => v == key).length > 0){
                            message = valObj.message[key]
                        }else{
                            message = {}
                            Object.keys(valObj.message).forEach(v => {
                                if(v.indexOf(`${key}.`) > -1){
                                    message[v] = valObj.message[v]
                                }
                            })
                        }
                        goCheck(key,obj[key],valObj.rule[key],message,valObj.scene[ctx.after])
                    }
                })
            }catch(err){
                if(err.msg){
                    throw new HttpException(err)
                }else{
                    throw new HttpException({
                        msg: '验证器可能书写有误',
                        errorCode: 20000,
                        statusCode: 500
                    })
                }
            }
        }
        return obj
    }

    /*
    * 返回成功的Json数据
    * data 实际数据
    * msg 状态消息
    * statusCode 状态码
    */
    showSuccess(data = [],msg = 'ok',statusCode = 200){
        return {
            msg,
            data,
            statusCode
        }
    }

    /*
    * 异常或错误返回的Json数据
    * msg 状态消息
    * errorCode 错误码
    * statusCode 状态码
    */
    ApiException(msg = '请求错误',errorCode = 30000,statusCode = 400){
        throw new HttpException({
            msg,
            errorCode,
            statusCode
        })
    }

    /*
    * 挂载ThinkRedis
    * ThinkRedis()
    */
    ThinkRedis(){
        return new ThinkRedis()
    }

    /*
    * 使用视图
    * url html文件路径
    * obj 给页面传递的参数，对象类型
    */
    View(url,obj){
        return {
            isView: true,
            url,
            obj
        }
    }

    /*
    * 使用模型
    */
    M(){}
}

module.exports = Controller