/*
 * @Author: zhangyu
 * @Date: 2021-04-17 18:38:00
 * @LastEditTime: 2021-04-29 12:48:10
 */

const HttpException = require('./exception.js')
const router = require('koa-router')
const Router = new router()
const fs = require('fs')

// 获取返回对象
const getReturnObj = (obj) => {
    if(obj instanceof Object){
        return {
            body: {
                msg: obj.msg,
                data: obj.data
            },
            status: obj.statusCode
        }
    }else{
        throw new HttpException({
            msg: '返回值不是一个对象类型',
            errorCode: 10000,
            statusCode: 500
        })
    }
}

// 动态执行控制器方法
const load = async (str,ctx) => {
    if(/\//g.test(str) && str.match(/\//g).length == 1){
        // controller目录下最多可分一层目录
        let before = str.split('/')[0]
        let after = str.split('/')[1]
        // 判断/前是否有.
        if(/\./g.test(before) && before.match(/\./g).length == 1){
            before = before.replace('.','/')
        }
        let controller = null
        try{
            controller = new (require(`@/controller/${before}.js`))
            ctx.before = before
            ctx.after = after
        }catch(e){
            throw new HttpException({
                msg: `路由配置第二个参数有误[${str}]`,
                errorCode: 20000,
                statusCode: 500
            })
        }
        try{
            let result = await controller[after](ctx)
            if(result.isView){
                return result
            }else{
                return getReturnObj(result)
            }
        }catch(err){
            throw new HttpException(err)
        }
    }else{
        throw new HttpException({
            msg: `路由配置第二个参数有误[${str}]`,
            errorCode: 20000,
            statusCode: 500
        })
    }
}

// 路由
class Route {
    // 判读路由是否重复
    isRepeat(url){
        if(Router.stack.some(v => v.path == url)){
            throw new HttpException({
                msg: `路由重复[${url}]`,
                errorCode: 20000,
                statusCode: 500
            })
        }
    }
    // GET路由
    get(url,str,middleware = null){
        this.isRepeat(url)
        Router.get(url,async (ctx) => {
            if(typeof middleware == 'function'){
                let result = await load(str,ctx)
                if(result.isView){
                    await ctx.render(result.url,result.obj)
                }
                middleware(ctx,(fileSrc = 'back') => {
                    if(result.isView){
                        ctx.redirect(fileSrc)
                    }else{
                        ctx.body = result.body
                        ctx.status = result.status
                    }
                },(msg = '请求错误',errorCode = 30000,statusCode = 400) => {
                    throw new HttpException({
                        msg,
                        errorCode,
                        statusCode
                    })
                })
            }else{
                let result = await load(str,ctx)
                if(result.isView){
                    await ctx.render(result.url,result.obj)
                }else{
                    ctx.body = result.body
                    ctx.status = result.status
                }
            }
        })
    }
    // POST路由
    post(url,str,middleware = null){
        this.isRepeat(url)
        Router.post(url,async (ctx) => {
            if(typeof middleware == 'function'){
                let result = await load(str,ctx)
                middleware(ctx,() => {
                    ctx.body = result.body
                    ctx.status = result.status
                },(msg = '请求错误',errorCode = 30000,statusCode = 400) => {
                    throw new HttpException({
                        msg,
                        errorCode,
                        statusCode
                    })
                })
            }else{
                let result = await load(str,ctx)
                ctx.body = result.body
                ctx.status = result.status
            }
        })
    }
    // 分组
    group(prefix,callback,middleware = null){
        callback({
            get: (url,str) => {
                this.get(`${prefix}${url}`,str,middleware)
            },
            post: (url,str)  => {
                this.post(`${prefix}${url}`,str,middleware)
            }
        })
    }
}

// 加载路由配置
fs.readdirSync('./route').forEach(file => {
    let fun = require(`@route/${file}`)
    fun(new Route)
})

module.exports = Router